@@ -12,8 +12,8 @@ from subprocess import check_output
 import errno
 import re
 from glob import glob
-from os import makedirs, chmod, environ, unlink
-from os.path import basename, exists, join, dirname, isdir, split, islink
+from os import makedirs, environ, unlink
+from os.path import basename, exists, join, dirname, isdir, islink
 from sys import argv
 from dhfortran import DEPENDS_SUBSTVARS, PKG_NAME_TPLS
 
@@ -41,30 +41,31 @@ def relpath(target, link):
     '../share/python-foo/foo.py'
     """
     t = target.split("/")
-    l = link.split("/")
+    lk = link.split("/")
     # remove any dup '//'
-    while '' in t:
-        t.remove('')
-    while ''  in l:
-        l.remove('')
-    while l and l[0] == t[0]:
-        del l[0], t[0]
-    return "/".join([".."] * (len(l) - 1) + t)
+    while "" in t:
+        t.remove("")
+    while "" in lk:
+        lk.remove("")
+    while lk and lk[0] == t[0]:
+        del lk[0], t[0]
+    return "/".join([".."] * (len(lk) - 1) + t)
 
 
-def move_file(fpath, dstdir):
-    """Move file to dstdir. Works with symlinks (including relative ones)."""
-    if isdir(fpath):
-        dname = split(fpath)[-1]
-        for fn in os.listdir(fpath):
-            move_file(join(fpath, fn), join(dstdir, dname))
+# def move_file(fpath, dstdir):
+#     """Move file to dstdir. Works with symlinks (including relative ones).
+#     TODO: relative_symlink undefined; fn unused, delete? """
+#     if isdir(fpath):
+#         dname = split(fpath)[-1]
+#         for fn in os.listdir(fpath):
+#             move_file(join(fpath, fn), join(dstdir, dname))
 
-    if islink(fpath):
-        dstpath = join(dstdir, split(fpath)[-1])
-        relative_symlink(os.readlink(fpath), dstpath)
-        os.remove(fpath)
-    else:
-        os.rename(fpath, dstdir)
+#     if islink(fpath):
+#         dstpath = join(dstdir, split(fpath)[-1])
+#         relative_symlink(os.readlink(fpath), dstpath)
+#         os.remove(fpath)
+#     else:
+#         os.rename(fpath, dstdir)
 
 
 def build_options(**options):
@@ -74,15 +75,16 @@ def build_options(**options):
         "package": [],
         "no_package": [],
         "match_packages": False,  # TODO Revist this decision
-        "write_log": False,
+        "write_log": True,
         "remaining_packages": False,
         "sourcedir": None,
+        "tmpdir": None,
         "DOPACKAGES": [],
     }
-    if options['arch'] == False:
-        options['arch'] = None
-    if type(options['package']) is str:
-        options['package'] = [ options['package'] ]
+    if "arch" in options and options["arch"] is False:
+        options["arch"] = None
+    if "package" in options and isinstance(options["package"], str):
+        options["package"] = [options["package"]]
     built_options = default_options
     built_options.update(options)
     return type("Options", (object,), built_options)
@@ -121,7 +123,7 @@ class DebHelper:
                     if line[0].isspace():  # Continuation
                         paragraphs[-1][field] += line.rstrip()
                         continue
-                    if not ":" in line:
+                    if ":" not in line:
                         raise Exception(
                             "Unable to parse line %i in debian/control: %s"
                             % (lineno, line)
@@ -181,14 +183,19 @@ class DebHelper:
             if options.remaining_packages and self.has_acted_on_package(binary_package):
                 continue
             if "package-type" in paragraph:
-                typ = package["package-type"]
+                typ = paragraph["package-type"]
             else:
                 typ = "deb"
+            if "section" in paragraph:
+                section = paragraph["section"]
+            else:
+                section = None
             pkg = {
                 "substvars": {},
                 "autoscripts": {},
                 "arch": paragraph["architecture"],
                 "type": typ,
+                "section": section,
             }
             if (
                 options.arch is False
@@ -208,9 +215,10 @@ class DebHelper:
                         "depends", ""
                     ) and substvar not in paragraph.get("recommends", ""):
                         cli.verbose_print(
-                            f"skipping package {binary_package} (missing {substvar} in Depends/Recommends)"
+                            f"skipping {binary_package}: (missing {substvar})"
                         )
                         continue
+
             # Operate on binary_package
             self.packages[binary_package] = pkg
 
@@ -221,11 +229,9 @@ class DebHelper:
 
     def has_acted_on_package(self, package):
         try:
-            with open(
-                "debian/{}.debhelper.log".format(self.package), encoding="utf-8"
-            ) as f:
+            with open(f"debian/{package}.debhelper.log", encoding="utf-8") as f:
                 for line in f:
-                    if line.strip() == self.command:
+                    if line.strip() == self.tool_name:
                         return True
         except IOError as e:
             if e.errno != errno.ENOENT:
@@ -239,52 +245,21 @@ class DebHelper:
         p = relpath(dest, src)
         if p == "":
             # src and dest are the same
-            return 
-        cli.verbose_print(f"DEBUG ln -s {s} {p}")
+            return
+        cli.verbose_print(f"ln -s {s} {p}")
         if islink(s):  # So symlink doesn't fail on idempotent
             unlink(s)
         os.symlink(p, s)
 
-
     # Policy says that if the link is all within one toplevel
     # directory, it should be relative. If it's between
     # top level directories, leave it absolute.
     # if not self.options.no_act:
     #    relative_symlink(dest, src)
 
-        
-    def filedoublearray(self, cfgfile, searchdirs=None, ErrHandler=Exception):
-        # Reads in the specified file, one line at a time. splits on words,
-        # and returns an array of arrays of the contents.
-        # If a value is passed in as the second parameter, then glob
-        # expansion is done in the directory specified by the parameter ("." is
-        # frequently a good choice).
-        # TODO: In compat  13+, it will do variable expansion (after splitting the lines
-        # into words)
-        result = []
-        with open(cfgfile, "r") as f:
-            lines = f.readlines()
-        if lines[0].startswith("#!"):
-            # Some config files are shell scripts that generate output, so ...
-            lines = check_output(['./' + cfgfile ]).decode("utf-8").split('\n')
-        for lineno, line in enumerate(lines, 1):
-            if line=='' or line.startswith("#"):
-                continue  # fragile
-            bits = line.strip().split()
-            dest = bits[1] if len(bits) > 1 else ""
-            matches = []
-            try:
-                x = self.find_files_double(searchdirs, dest, bits[0])
-                result += x
-            except Exception as ex:
-                badline = f"Bad line {lineno} in {cfgfile}"
-                cli.warning(badline)
-        # TODO finish error parsing
-        return result
-
-    def find_files_double(self, searchdirs, dest, *files):
-        """ Search for each file in searchdirs, return list of pairs (file, dest)
-        Recurse into directories """
+    def find_files(self, searchdirs, pkg, dest, *files):
+        """Search for each file in searchdirs, return list of tuples (pkg,file, dest)
+        Recurse into directories"""
         result = []
         for f in files:
             if not f:
@@ -295,10 +270,10 @@ class DebHelper:
                 for p in glob(nm, root_dir=s):
                     if isdir(f"{s}/{p}"):
                         subdest = basename(p) if dest == "" else f"{dest}/{basename(p)}"
-                        x = self.find_files_double([f"{s}/{p}"], subdest, "*")
+                        x = self.find_files(pkg, subdest, [f"{s}/{p}"], "*")
                         matches += x
                     else:
-                        matches.append([f"{s}/{p}", dest])
+                        matches.append((pkg, f"{s}/{p}", dest))
             if matches:
                 result += matches
             else:
@@ -324,10 +299,10 @@ class DebHelper:
 
     def doit(self, *args, **kwargs):
         """Execute a command given as a list"""
-        cli.verbose_print(f"DEBUG doit({args},{kwargs})")
+        cli.verbose_print(f"doit({args},{kwargs})")
         if self.options.no_act:
             return
-        result = check_output(*args)
+        check_output(*args)
         # TODO
 
     def is_udeb(self, package):
@@ -416,60 +391,21 @@ class DebHelper:
                 with open(fn, "w", encoding="utf-8") as fp:
                     fp.write(data)
 
-    def clone2(fname: str, suffix) -> str:
-        """Clone a file, returning new filename
-
-        typ in ('lib','cmake','pkgconf')
-        either flavor or vendor must be set
-
-        if rewrite is defined its a function:
-        rewrite(oldname,newname,cb_dict)
-        """
-        # Rename cmake file. Should be named xxx.cmake
-        # TODO Use pathlib
-        pathbits = fname.split("/")
-        basename = pathbits[-1]
-        if len(pathbits) == 1:
-            pth = "/".join(pathbits[:-1])
-        else:
-            pth = ""
-        name, typ = ".".join(basename.split(".")[:-1]), basename.split(".")[-1]
-        newname = pth + name + "-" + suffix + "." + typ
-        if "NO_ACT" in os.environ:
-            verbose_print(f"clone {fname} as {newname}")
-        else:
-            if rewrite_fn:
-                rewrite_fn(fname, newname, cb_dict)
-            else:
-                with open(fname, "r") as f:
-                    with open(newname, "w") as n:
-                        n.write(f.read)
-        return newname
-
-    def pkgfile(self, package, suffix):
+    def pkgfile(self, package: str, suffix: str) -> str | None:
         """Incomplete clone of /usr/share/perl5/Debian/Debhelper/Dh_Lib.pm version"""
         hostarch, hostos = "FIXME", "FIXME"
-        if "name" in self.options.__dict__:  # TODO, not populated
-            name = self.options.name
-            searchpath = [
-                f"debian/{package}.{name}.{suffix}.{hostarch}",
-                f"debian/{package}.{name}.{suffix}.{hostos}",
-                f"debian/{package}.{name}.{suffix}",
-                f"debian/{name}.{suffix}",
-            ]
-        else:
-            searchpath = [
-                f"debian/{package}.{suffix}.{hostarch}",
-                f"debian/{package}.{suffix}.{hostos}",
-                f"debian/{package}.{suffix}",
-                f"debian/{suffix}",
-            ]
+        searchpath = [
+            f"debian/{package}.{suffix}.{hostarch}",
+            f"debian/{package}.{suffix}.{hostos}",
+            f"debian/{package}.{suffix}",
+            f"debian/{suffix}",
+        ]
         for s in searchpath:
             if exists(s):
                 return s
         return None
 
-    def generated_file(self, package, filename):
+    def generated_file(self, package: str, filename: str):
         # Generated files are cleaned by dh_clean AND dh_prep
         # - Package can be set to "_source" to generate a file relevant
         #   for the source package (the meson build does this atm.).
@@ -489,47 +425,105 @@ class DebHelper:
             for src in installed:
                 print(src, file=f)
 
-    def process_and_move_files(self, *files):
-        for pkg in self.packages:
-            if self.options.package is not None and pkg not in self.options.package:
-                continue
-            if self.is_udeb(pkg):
-                continue
-            if  pkg not in self.options.DOPACKAGES:
-                continue
-            if self.options.firstpackage and pkg != self.options.firstpackage:
-                cli.verbose_print(
-                    f"DEBUG: checking package FIRSTPACKAGE {self.options.firstpackage} only "
-                )
-                continue
+    def skip_package(self, pkg) -> bool:
+        """Do we skip this package or process it ?"""
+        if self.options.package is not None and pkg not in self.options.package:
+            return True
+        if self.is_udeb(pkg):
+            return True
+        if pkg not in self.options.DOPACKAGES:
+            return True
+        if self.options.firstpackage and pkg != self.options.firstpackage:
+            cli.debug_print(
+                f"checking package FIRSTPACKAGE {self.options.firstpackage} only "
+            )
+        return False
 
-            # Look at the install files for all packages to handle
-            # list-missing/fail-missing, but skip really installing for
-            # packages that are not being acted on.
+    def generate_pkg_filelist_from_cfgfile(
+        self, searchdirs: list[str], pkg: str
+    ) -> list[tuple[str, str, str]]:
+        """Generate a list of files to work on
+        @return list of tuples (package, filename, dest_dir)
+        """
 
-            config_file = self.pkgfile(pkg, self.suffix)  # eg  "debian/pkg.fortran_mod"
-            self.packages[pkg]["config_file"] = config_file
-            search_dirs = [self.options.sourcedir, "." ]
-
-            install_files =  self.find_files_double(search_dirs, "", *files)
-
-            if config_file:
-                install_files += self.filedoublearray(config_file, search_dirs)
+        def expand_variables(bits: list[str]) -> list[str]:
+            """ Helper fn. Do variable expansion on filepaths """
+            new_bits = []
+            for fpath in bits:
+                new_fpath = []
+                for elm in fpath.split('/'):
+                    if elm.startswith("$"):
+                        # TODO
+                        pass
+                    new_fpath.append(elm)
+                new_bits.append("/".join(new_fpath))
+            return new_bits                                   
                 
-            # TODO Skip excluded patterns in install files
+        # Reads in the specified file, one line at a time. splits on words
+        # If a value is passed in as the second parameter, then glob
+        # expansion is done in the directory specified by the parameter ("." is
+        # frequently a good choice).
+        # TODO: In compat  13+, it will do variable expansion (after splitting the lines
+        # into words)
+        worklist = []
+        cfgfile = self.packages[pkg]["config_file"]
+        with open(cfgfile, "r") as f:
+            lines = f.readlines()
+        if lines[0].startswith("#!"):
+            # Some config files are shell scripts that generate output, so ...
+            lines = check_output(["./" + cfgfile]).decode("utf-8").split("\n")
+        for lineno, line in enumerate(lines, 1):
+            if line == "" or line.startswith("#"):
+                continue  # fragile
+            bits = line.strip().split()
+            bits = expand_variables(bits)
+            dest = bits[1] if len(bits) > 1 else ""
+            try:
+                worklist += self.find_files(searchdirs, pkg, dest, bits[0])
+            except Exception:
+                badline = f"Bad line {lineno} in {cfgfile}"
+                cli.warning(badline)
+        # TODO finish error parsing
+        return worklist
 
-            self.log_installed_files(pkg, [ i[0]  for i in  install_files])
-            for f in install_files:
-                dest = f[1] if len(f) > 1 else None
-                target_dir = self.process_file(pkg, f[0], f"debian/{pkg}", dest)
+    def process_and_move_files(self, search_dirs, *cmdline_files):
+        """The bulk of the work.
+        subclass provides methods can_work_automatically(),
+           autogenerate_pkg_filelist(), process_file()
+        """
+
+        # Worklist: list of (pkg, file, dest) tuples
+        worklist = []
+        if self.can_work_automatically(*cmdline_files):
+            worklist += self.autogenerate_pkg_filelist(search_dirs)
+        else:
+            for pkg in self.config_file_pkgs:
+                if self.skip_package(pkg):
+                    continue
+                worklist += self.generate_pkg_filelist_from_cfgfile(search_dirs, pkg)
+            if len(cmdline_files) > 0:
+                # TODO . glob breakdown
+                pkg = self.options.package
+                if pkg is not None and not self.skip_package(pkg):
+                    for f in cmdline_files:
+                        worklist += self.find_files(search_dirs, pkg, "", f)
+
+        # Look at the install files for all packages to handle
+        # list-missing/fail-missing, but skip really installing for
+        # packages that are not being acted on.
+
+        # TODO Skip excluded patterns in install files
+
+        for pkg, fil, dest in worklist:
+            self.log_installed_files(pkg, [fil])
+            self.process_file(pkg, fil, f"debian/{pkg}", dest)
 
         if not self.options.write_log:
             return
         for package, _ in self.packages.items():
-            with open(
-                "debian/{}.debhelper.log".format(package), "a", encoding="utf-8"
-            ) as f:
-                f.write(self.command + "\n")
+            with open(f"debian/{package}.debhelper.log", "a", encoding="utf-8") as f:
+                f.write(self.tool_name + "\n")
+        
 
     def save(self):
         self.save_substvars()
@@ -540,12 +534,24 @@ class DebHelper:
 class DhFortranHelper(DebHelper):
     """dhfortran-specific stuff"""
 
-    def __init__(self, options, suffix, tool_name):
+    def __init__(self, options, suffix: str, tool_name: str):
         # TODO rework these
         self.command = {"fortran": "dh-fortran"}
         self.tool_name = tool_name
         self.suffix = suffix
         super().__init__(options, "fortran")
+        for pkg in self.packages:
+            config_file = self.pkgfile(pkg, self.suffix)
+            self.packages[pkg]["config_file"] = config_file
+        self.libs_pkgs = [
+            p for p in self.packages if self.packages[p]["section"] == "libs"
+        ]
+        self.libdevel_pkgs = [
+            p for p in self.packages if self.packages[p]["section"] == "libdevel"
+        ]
+        self.config_file_pkgs = [
+            p for p in self.packages if self.packages[p]["config_file"]
+        ]
 
 
 if __name__ == "__main__":
